@@ -1,5 +1,6 @@
 import os
 import re
+import shutil
 import socket
 import subprocess
 import time
@@ -19,6 +20,7 @@ from .utils import (
     SlurmRemote,
     SSHConfig,
     T,
+    qualified,
     randname,
     with_control_file,
     yn,
@@ -262,6 +264,9 @@ class milatools:
         # (defaults to "code" or the value of $MILATOOLS_CODE_COMMAND)
         command: Option = None
 
+        # Whether the server should persist or not
+        persist: Option & bool = default(False)
+
         if command is None:
             command = os.environ.get("MILATOOLS_CODE_COMMAND", "code")
 
@@ -269,29 +274,44 @@ class milatools:
         here = Local()
 
         cnode = _find_allocation(remote)
-        node_name, proc = cnode.ensure_allocation()
+        if persist:
+            cnode = cnode.persist()
+        data, proc = cnode.ensure_allocation()
+
+        node_name = data["node_name"]
 
         if not path.startswith("/"):
             # Get $HOME because we have to give the full path to code
             home = remote.home()
             path = "/".join([home, path])
 
-        time.sleep(1)
-        if "." not in node_name and not node_name.endswith(".server.mila.quebec"):
-            node_name = f"{node_name}.server.mila.quebec"
         try:
-            here.run(
-                command,
-                "-nw",
-                "--remote",
-                f"ssh-remote+{node_name}",
-                path,
-            )
+            while True:
+                here.run(
+                    shutil.which(command),
+                    "-nw",
+                    "--remote",
+                    f"ssh-remote+{qualified(node_name)}",
+                    path,
+                )
+                print(
+                    "The editor was closed. Reopen it with <Enter>"
+                    " or terminate the process with <Ctrl+C>"
+                )
+                input()
+
         except KeyboardInterrupt:
-            pass
-        if proc is not None:
-            proc.kill()
-        print(f"Ended session on '{node_name}'")
+            if not persist:
+                if proc is not None:
+                    proc.kill()
+                print(f"Ended session on '{node_name}'")
+
+        if persist:
+            print(f"This allocation is persistent and is still active.")
+            print(f"To reconnect to this node:")
+            print(T.bold(f"  mila code {path} --node {node_name}"))
+            print(f"To kill this allocation:")
+            print(T.bold(f"  ssh mila scancel {data['jobid']}"))
 
     class serve:
         """Start services on compute nodes and forward them to your local machine."""
@@ -403,7 +423,7 @@ class milatools:
                     "conda": "conda install -y jupyterlab",
                     "pip": "pip install jupyterlab",
                 },
-                command="jupyter lab --sock {sock}",
+                command="jupyter lab --sock {sock} {path}",
                 # command="jupyter lab --ip {host} --port 0",
                 token_pattern=r"\?token=([a-f0-9]+)",
             )
@@ -425,7 +445,7 @@ class milatools:
                     "conda": "conda install -y jupyter",
                     "pip": "pip install jupyter",
                 },
-                command="jupyter notebook --sock {sock}",
+                command="jupyter notebook --sock {sock} {path}",
                 # command="jupyter notebook --ip {host} --port 0",
                 token_pattern=r"\?token=([a-f0-9]+)",
             )
@@ -664,7 +684,7 @@ def _find_allocation(remote):
         exit("ERROR: --node, --job and --alloc are mutually exclusive")
 
     if node is not None:
-        node_name = f"{node}.server.mila.quebec"
+        node_name = qualified(node)
         return Remote(node_name)
 
     elif job is not None:
